<!DOCTYPE html>
<html lang="en">
    <head>
        <title>test-url</title>
    </head>
    <body>
        Empty: {{ url_for("", **kwargs) }}<br/>
        Dot: {{ url_for(".", **kwargs) }}<br/>
        Dot page: {{ url_for(".", "my-page", **kwargs) }}<br/>
        Slash: {{ url_for("/", **kwargs) }}<br/>
        Page: {{ url_for("my-page", **kwargs) }}<br/>
        Slash page: {{ url_for("/my-page", **kwargs) }}<br/>
        Query: {{ url_for("my-page", foo='1', bar='test with space', **kwargs) }}<br/>
    </body>
</html>
