table.fixedHeader-floating,
table.fixedHeader-locked {
  position: relative !important;
  background-color: var(--dt-html-background);
  background-color: var(--dt-html-background);
}

@media print {
  table.fixedHeader-floating,
  table.fixedHeader-locked {
    display: none;
  }
}
