{# def header="Language selection" #}
<li>
  <h6 class="dropdown-header">{{ header }}</h6>
</li>
{% set cur_locale = get_translation().locale %}
{% for locale in list_available_locales() %}
  <li>
    <button aria-pressed="{{ 'true' if cur_locale == locale else 'false' }}"
            class="dropdown-item btn-locale{{ ' active' if cur_locale == locale else '' }}"
            data-locale="{{ locale.language }}"
            type="button">{{ locale.display_name.capitalize() }}</button>
  </li>
{% endfor %}
