<!DOCTYPE html>
<html lang="en">
    <head>
        <title>test-form</title>
    </head>
    <body>
        {% if form.error_message %}
        <p>
            {{ form.error_message }}
        </p>
        {% endif %}
        <form method="post" action="/">
            <Fields form={{ form }} />
        </form>
    </body>
</html>
