<!DOCTYPE html>
<html lang="en">
    <head>
        <title>test-flash</title>
    </head>
    <body>
        <Flash messages={{ get_flashed_messages() }} class="mb-2" style="min-height:100px"/>
    </body>
</html>