<!DOCTYPE html>
<html lang="{{ get_translation().locale }}">
    <head>
        <title>test-jinja2</title>
        {{ catalog.render_assets() }}
    </head>
    <body>
        <!-- Selector -->
        <div class="dropdown">
            <button class="btn btn-secondary dropdown-toggle" type="button" data-bs-toggle="dropdown" aria-expanded="false">
                Dropdown button
            </button>
            <ul class="dropdown-menu">
                <LocaleSelection />
            </ul>
        </div>
        {{ get_language_name(get_translation().locale) }}<br/>
        {% trans %}Some text to translate{% endtrans %}<br/>
        {{ my_datetime | format_datetime(format='full') }}<br/>
        {{ my_date | format_date(format='full') }}
    </body>
</html>
