/* Make Multi-wrapper similar to other bootstrap */
.multi-wrapper {
    border: 1px solid #ced4da;
    border-radius: 0.375rem;
    background: #ffffff;
}

.multi-wrapper .search-input,
.multi-wrapper .non-selected-wrapper,
.multi-wrapper .selected-wrapper {
    background: none;
    padding: 0.375rem 0.75rem;
    overflow-y: auto;
}

.multi-wrapper .header {
    padding: 1px;
}

.multi-wrapper .item {
    padding: 1px;
    white-space: nowrap;
    text-overflow: ellipsis;
    overflow: hidden;
}
