/* Remove default margin from layout */
div.dt-container div.dt-layout-row {
    margin: 0;
}

/* Remove default seperator */
div.dt-container.dt-empty-footer .dt-scroll-body {
    border-bottom: 0;
}

/* Show hide Action button */
.dt-container tr .btn-hover {
    opacity: 0;
}

.dt-container tr:hover .btn-hover {
    opacity: 1;
}

/*
* Hide responsive column "none" during loading.
*/
.dt-container th.none {
    display: none;
}

/* Fix background color of button collection in DarkMode */
div.dt-button-collection {
    background-color: var(--bs-body-bg);
    color: var(--bs-body-color);
    border-color: var(--bs-border-color-translucent);
}

/* Css for responsive details render tableHidden */
table.dtr-details .dtr-title-cell {
  width: 15%;
  white-space: nowrap;
  vertical-align: top;
  padding: 8px 10px;
}

table.dtr-details .dtr-data-cell {
  width: 85%;
  white-space: normal;
  word-break: break-word;
  padding: 8px 10px;
}
