<!DOCTYPE html>
<html lang="en">
    <head>
        <title>test-jinja2</title>
    </head>
    <body>
        var1: {{ var1 }}<br/>
        var2: {{ var2 }}<br/>
        const1: {{ const1 }}<br/>
    </body>
</html>
